require_dependency 'mailer'

module MailerSubscriptionPatch

    def self.included(base)
        if Redmine::VERSION::MAJOR > 1
            base.send(:include, Redmine2InstanceMethods)
        else
            base.send(:include, Redmine1InstanceMethods)
        end
        base.class_eval do
            unloadable

            alias_method_chain :attachments_added, :subscribers

            if Redmine::VERSION::MAJOR < 2
                alias_method_chain :issue_add,  :subscribers
                alias_method_chain :issue_edit, :subscribers
            end
        end
    end

    module Redmine2InstanceMethods

        def attachments_added_with_subscribers(attachments)
            message = attachments_added_without_subscribers(attachments)

            notified_users = []

            container = attachments.first.container
            case container
            when Project
                if container.module_enabled?(:subscription)
                    subscribers    = ProjectSubscriber.find_all_by_project_id(container.id)
                    notified_users = subscribers.select{ |subscriber| subscriber.subscribed_to?(:files) &&
                                                                      subscriber.user.allowed_to?(:view_files, container) &&
                                                                      subscriber.user.allowed_to?(:subscribe_to_project, container)
                                                       }.collect(&:user)
                end
            when Version
                if container.project.module_enabled?(:subscription)
                    subscribers    = ProjectSubscriber.find_all_by_project_id(container.project.id)
                    notified_users = subscribers.select{ |subscriber| subscriber.subscribed_to?(:files) &&
                                                                      subscriber.user.allowed_to?(:view_files, container.project) &&
                                                                      subscriber.user.allowed_to?(:subscribe_to_project, container.project)
                                                       }.collect(&:user)
                end
            end

            unless notified_users.empty?
                if @author && @author.logged? && @author.pref.no_self_notified
                    notified_users.reject!{ |user| user == @author }
                end

                message.to = message.to + notified_users.collect(&:mail)
            end

            message
        end

        def version_closed(version)
            notified = []
            if version.project.module_enabled?(:subscription)
                subscribers = ProjectSubscriber.find_all_by_project_id(version.project.id)
                notified    = subscribers.select{ |subscriber| subscriber.subscribed_to?(:releases) &&
                                                               subscriber.user.allowed_to?(:subscribe_to_project, version.project) &&
                                                               version.visible?(subscriber.user)
                                                }.collect{ |subscriber| subscriber.user.mail }
            end

            @wiki_url = nil
            if version.project.wiki && !version.wiki_page_title.blank?
                wiki = version.project.wiki.find_page(version.wiki_page_title)
                if wiki
                    @wiki_url = url_for(:controller => 'wiki',
                                        :action     => 'show',
                                        :project_id => version.project,
                                        :id         => Wiki.titleize(version.wiki_page_title))
                end
            end

            redmine_headers 'Project'    => version.project.identifier,
                            'Version'    => version.name,
                            'Version-Id' => version.id
            message_id version
            references version
            @version     = version
            @project_url = url_for(:controller => 'projects', :action => 'show', :id => version.project)
            @version_url = url_for(:controller => 'versions', :action => 'show', :id => version)
            mail :to      => notified,
                 :subject => "[#{version.project.name}] " + l(:mail_subject_version_closed, :version => version.name)
        end

        def repository_added(repository)
            notified = []
            if repository.project.module_enabled?(:subscription)
                subscribers = ProjectSubscriber.find_all_by_project_id(repository.project.id)
                notified    = subscribers.select{ |subscriber| subscriber.subscribed_to?(:repository) &&
                                                               subscriber.user.allowed_to?(:subscribe_to_project, repository.project) &&
                                                               subscriber.user.allowed_to?(:view_changesets, changeset.project)
                                                }.collect{ |subscriber| subscriber.user.mail }
            end

            redmine_headers 'Project'       => repository.project.identifier,
                            'SCM-Type'      => repository.type,
                            'Repository-Id' => repository.id
            message_id repository
            references repository
            @repository_url = url_for(:controller => 'repositories', :action => 'show', :id => repository.project)
            mail :to      => notified,
                 :subject => "[#{repository.project.name}]" + l(:mail_subject_repository_added)
        end

        def changeset_added(changeset)
            notified = []
            if changeset.project.module_enabled?(:subscription)
                subscribers = ProjectSubscriber.find_all_by_project_id(changeset.project.id)
                notified    = subscribers.select{ |subscriber| subscriber.subscribed_to?(:repository) &&
                                                               subscriber.user.allowed_to?(:subscribe_to_project, changeset.project) &&
                                                               subscriber.user.allowed_to?(:view_changesets, changeset.project)
                                                }.collect{ |subscriber| subscriber.user.mail }
            end

            if changeset.user && changeset.user.logged? && changeset.user.pref.no_self_notified
                notified.reject!{ |user| user == changeset.user }
            end

            redmine_headers 'Project'     => changeset.project.identifier,
                            'Revision'    => changeset.identifier,
                            'Revision-Id' => changeset.id
            title = "[#{changeset.project.name}] #{l(:label_revision)} #{changeset.format_identifier}"
            title << ": #{changeset.short_comments}" unless changeset.short_comments.blank?
            @author       = changeset.user
            @changeset    = changeset
            @revision_url = url_for(:controller => 'repositories', :action => 'revision', :id => changeset.project, :rev => changeset.identifier)
            @diff_url     = url_for(:controller => 'repositories', :action => 'diff', :id => changeset.project, :rev => changeset.identifier, :path => "")
            mail :to => notified,
                 :subject => title
        end

        def board_added(board)
            notified = []
            if board.project.module_enabled?(:subscription)
                subscribers = ProjectSubscriber.find_all_by_project_id(board.project.id)
                notified    = subscribers.select{ |subscriber| subscriber.subscribed_to?(:forums) &&
                                                               subscriber.user.allowed_to?(:subscribe_to_project, board.project) &&
                                                               board.visible?(subscriber.user)
                                                }.collect{ |subscriber| subscriber.user.mail }
            end

            redmine_headers 'Project'  => board.project.identifier,
                            'Board'    => board.name,
                            'Board-Id' => board.name
            @board     = board
            @board_url = url_for(:controller => 'boards', :action => 'show', :project_id => board.project, :id => board)
            mail :to      => notified,
                 :subject => "[#{board.project.name}] #{l(:mail_subject_board_added)}: #{board.name}"
        end

    end

    module Redmine1InstanceMethods

        def attachments_added_with_subscribers(attachments)
            attachments_added_without_subscribers(attachments)

            notified_users = []
            container = attachments.first.container
            case container
            when Project
                if container.module_enabled?(:subscription)
                    subscribers    = ProjectSubscriber.find_all_by_project_id(container.id)
                    notified_users = subscribers.select{ |subscriber| subscriber.subscribed_to?(:files) &&
                                                                      subscriber.user.allowed_to?(:view_files, container) &&
                                                                      subscriber.user.allowed_to?(:subscribe_to_project, container) }
                end
            when Version
                if container.project.module_enabled?(:subscription)
                    subscribers    = ProjectSubscriber.find_all_by_project_id(container.project.id)
                    notified_users = subscribers.select{ |subscriber| subscriber.subscribed_to?(:files) &&
                                                                      subscriber.user.allowed_to?(:view_files, container.project) &&
                                                                      subscriber.user.allowed_to?(:subscribe_to_project, container.project) }
                end
            end
            unless notified_users.empty?
                recipients @recipients + notified_users.collect{ |subscriber| subscriber.user.mail }
            end
        end

        def issue_add_with_subscribers(issue)
            issue_add_without_subscribers(issue)

            if issue.project.module_enabled?(:subscription)
                subscribers    = ProjectSubscriber.find_all_by_project_id(issue.project.id)
                notified_users = subscribers.select{ |subscriber| subscriber.subscribed_to?(:new_issues) &&
                                                                  subscriber.user.allowed_to?(:subscribe_to_project, issue.project) &&
                                                                  issue.visible?(subscriber.user) }.collect{ |subscriber| subscriber.user.mail }
                notified_users -= @recipients
                notified_users -= @cc
                recipients @recipients + notified_users
            end
        end

        def issue_edit_with_subscribers(journal)
            issue_edit_without_subscribers(journal)

            status = journal.new_status
            if status && status.is_closed?
                issue = journal.journalized.reload
                if issue.project.module_enabled?(:subscription)
                    subscribers    = ProjectSubscriber.find_all_by_project_id(issue.project.id)
                    notified_users = subscribers.select{ |subscriber| subscriber.subscribed_to?(:closed_issues) &&
                                                                      subscriber.user.allowed_to?(:subscribe_to_project, issue.project) &&
                                                                      issue.visible?(subscriber.user) }.collect{ |subscriber| subscriber.user.mail }
                    notified_users -= @recipients
                    notified_users -= @cc
                    recipients @recipients + notified_users
                end
            end
        end

        def version_closed(version)
            notified = []
            if version.project.module_enabled?(:subscription)
                subscribers = ProjectSubscriber.find_all_by_project_id(version.project.id)
                notified    = subscribers.select{ |subscriber| subscriber.subscribed_to?(:releases) &&
                                                               subscriber.user.allowed_to?(:subscribe_to_project, version.project) &&
                                                               version.visible?(subscriber.user) }.collect{ |subscriber| subscriber.user.mail }
            end

            wiki_url = nil
            if version.project.wiki && !version.wiki_page_title.blank?
                wiki = version.project.wiki.find_page(version.wiki_page_title)
                if wiki
                    url_for(:controller => 'wiki', :action => 'show', :project_id => version.project, :id => Wiki.titleize(version.wiki_page_title))
                end
            end

            redmine_headers 'Project'    => version.project.identifier,
                            'Version'    => version.name,
                            'Version-Id' => version.id
            message_id version
            recipients notified
            subject "[#{version.project.name}] " + l(:mail_subject_version_closed, :version => version.name)
            body :version     => version,
                 :project_url => url_for(:controller => 'projects', :action => 'show', :id => version.project),
                 :version_url => url_for(:controller => 'versions', :action => 'show', :id => version),
                 :wiki_url    => wiki_url
            render_multipart('version_closed', body)
        end

        def repository_added(repository)
            notified = []
            if repository.project.module_enabled?(:subscription)
                subscribers = ProjectSubscriber.find_all_by_project_id(repository.project.id)
                notified    = subscribers.select{ |subscriber| subscriber.subscribed_to?(:repository) &&
                                                               subscriber.user.allowed_to?(:subscribe_to_project, repository.project) &&
                                                               subscriber.user.allowed_to?(:view_changesets, changeset.project)
                                                }.collect{ |subscriber| subscriber.user.mail }
            end

            redmine_headers 'Project'       => repository.project.identifier,
                            'SCM-Type'      => repository.type,
                            'Repository-Id' => repository.id
            recipients notified
            subject "[#{repository.project.name}]" + l(:mail_subject_repository_added)
            body :repository_url => url_for(:controller => 'repositories', :action => 'show', :id => repository.project)
            render_multipart('repository_added', body)
        end

        def changeset_added(changeset)
            notified = []
            if changeset.project.module_enabled?(:subscription)
                subscribers = ProjectSubscriber.find_all_by_project_id(changeset.project.id)
                notified    = subscribers.select{ |subscriber| subscriber.subscribed_to?(:repository) &&
                                                               subscriber.user.allowed_to?(:subscribe_to_project, changeset.project) &&
                                                               subscriber.user.allowed_to?(:view_changesets, changeset.project)
                                                }.collect{ |subscriber| subscriber.user.mail }
            end

            redmine_headers 'Project'     => changeset.project.identifier,
                            'Revision'    => changeset.identifier,
                            'Revision-Id' => changeset.id
            @author = changeset.user
            recipients notified
            title = "[#{changeset.project.name}] #{l(:label_revision)} #{changeset.format_identifier}"
            title << ": #{changeset.short_comments}" unless changeset.short_comments.blank?
            subject title
            body :changeset    => changeset,
                 :revision_url => url_for(:controller => 'repositories', :action => 'revision', :id => changeset.project, :rev => changeset.identifier),
                 :diff_url     => url_for(:controller => 'repositories', :action => 'diff', :id => changeset.project, :rev => changeset.identifier, :path => "")
            render_multipart('changeset_added', body)
        end

        def board_added(board)
            notified = []
            if board.project.module_enabled?(:subscription)
                subscribers = ProjectSubscriber.find_all_by_project_id(board.project.id)
                notified    = subscribers.select{ |subscriber| subscriber.subscribed_to?(:forums) &&
                                                               subscriber.user.allowed_to?(:subscribe_to_project, board.project) &&
                                                               board.visible?(subscriber.user) }.collect{ |subscriber| subscriber.user.mail }
            end

            redmine_headers 'Project'  => board.project.identifier,
                            'Board'    => board.name,
                            'Board-Id' => board.name
            recipients notified
            subject "[#{board.project.name}] #{l(:mail_subject_board_added)}: #{board.name}"
            body :board     => board,
                 :board_url => url_for(:controller => 'boards', :action => 'show', :project_id => board.project, :id => board)
            render_multipart('board_added', body)
        end

    end

end
